@@ -3,7 +3,7 @@ use Any::Moose;
 extends 'Path::Dispatcher::Rule';
 
 has regex => (
-    is       => 'rw',
+    is       => 'ro',
     isa      => 'RegexpRef',
     required => 1,
 );
@@ -12,19 +12,17 @@ sub _match {
     my $self = shift;
     my $path = shift;
 
-    return unless my @matches = $path->path =~ $self->regex;
+    return unless my @positional = $path->path =~ $self->regex;
 
-    # if $' is in the program at all, then it slows down every single regex
-    # we only want to include it if we have to
-    if ($self->prefix) {
-        return \@matches, eval q{$'};
+    my %named = $] > 5.010 ? eval q{%+} : ();
+
+    return {
+        positional_captures => \@positional,
+        named_captures      => \%named,
+        ($self->prefix ? (leftover => eval q{$'}) : ()),
     }
-
-    return \@matches;
 }
 
-sub readable_attributes { shift->regex }
-
 __PACKAGE__->meta->make_immutable;
 no Any::Moose;
 
@@ -40,7 +38,7 @@ Path::Dispatcher::Rule::Regex - predicate is a regular expression
 
     my $rule = Path::Dispatcher::Rule::Regex->new(
         regex => qr{^/comment(s?)/(\d+)$},
-        block => sub { display_comment($2) },
+        block => sub { display_comment(shift->pos(2)) },
     );
 
 =head1 DESCRIPTION
@@ -53,9 +51,8 @@ Rules of this class use a regular expression to match against the path.
 
 The regular expression to match against the path. It works just as you'd expect!
 
-The results are the capture variables (C<$1>, C<$2>, etc) and when the
-resulting L<Path::Dispatcher::Match> is executed, the codeblock will see these
-values. C<$`>, C<$&>, and C<$'> are not (yet) restored.
+The capture variables (C<$1>, C<$2>, etc) will be available in the match
+object as C<< ->pos(1) >> etc. C<$`>, C<$&>, and C<$'> are not restored.
 
 =cut
 
